SystemOrganization addCategory: #VersionChangesCalculator!

Object subclass: #ChangeCriteria
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

ChangeCriteria subclass: #ChangeByCyclomaticComplexity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByCyclomaticComplexity methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass weightedMethodCount! !

ChangeCriteria subclass: #ChangeByLinesNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByLinesNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfLinesOfCode! !

ChangeCriteria subclass: #ChangeByMethodsNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByMethodsNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfMethods! !

!ChangeCriteria methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ 0! !

Object subclass: #ClassVolatility
	instanceVariableNames: 'className volatility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
< cVol 
	^ volatility < cVol volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
<= cVol 
	^ volatility <= cVol volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
= cVol 
	^ className = cVol className & volatility = cVol volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
> cVol 
	^ volatility > cVol volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
>= cVol 
	^ volatility >= cVol volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
className
	^ className! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
className: cName 
	className := cName! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
volatility
	^ volatility! !

!ClassVolatility methodsFor: 'as yet unclassified' stamp: ''!
volatility: vol 
	volatility := vol! !

Object subclass: #MovingAverageCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesMovingAverages: classesDict withPeriod1: period1 andPeriod2: period2 saveToFile: fileName 
	| stream movAvg1 movAvg2 |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Calculating Moving Averages...';
		 cr;
		 cr.
	classesDict
		keysDo: [:className | 
			| classValues versionNumsList valuesList |
			stream nextPutAll: 'Class Name: ' , className;
				 cr.
			classValues := classesDict at: className.
			versionNumsList := LinkedList new.
			valuesList := LinkedList new.
			classValues
				do: [:versionInfo | 
					versionNumsList add: versionInfo versionNum.
					valuesList add: versionInfo value].
			stream nextPutAll: 'Version Nums: , ' , versionNumsList asCommaString;
				 cr.
			stream nextPutAll: 'Values: , ' , valuesList asCommaString;
				 cr.
			movAvg1 := self calculateMovingAverage: classValues withPeriod: period1.
			movAvg2 := self calculateMovingAverage: classValues withPeriod: period2.
			stream nextPutAll: 'Mov Avg ' , period1 asString , ': , ' , movAvg1 asCommaString;
				 cr.
			stream nextPutAll: 'Mov Avg ' , period2 asString , ': , ' , movAvg2 asCommaString;
				 cr;
				 cr].
	stream close! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMovingAverage: versionsInfo withPeriod: period 
	| movingAverages periodList periodCount totalPeriodValue |
	periodCount := 1.
	totalPeriodValue := 0.
	periodList := LinkedList new: 3.
	movingAverages := LinkedList new.
	versionsInfo
		do: [:versInfo | 
			| value |
			value := versInfo value.
			periodCount < period
				ifTrue: [periodCount := periodCount + 1.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value].
					periodList add: value]
				ifFalse: [periodList add: value.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value.
							totalPeriodValue := totalPeriodValue - periodList removeFirst].
					movingAverages add: (totalPeriodValue / period) asFloat]].
	^ movingAverages! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
createLineChart: values withMAValues: movAvgValues1 andMAValues2: movAvgValues2 
	| diag1 diag2 diag3 compDiag |
	diag1 := ESDiagramRenderer new lineDiagram y: #yourself;
				 models: values;
				 defaultColor: Color red;
				 yourself.
	diag2 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color green;
				 models: movAvgValues2;
				 yourself.
	diag3 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color blue;
				 models: movAvgValues1;
				 yourself.
	compDiag := ESDiagramRenderer new.
	compDiag compositeDiagram add: diag1;
		 add: diag2;
		 add: diag3;
		 preferredAxisMaxY: 45.
	^ compDiag! !

Object subclass: #VersionChangesCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionChanges: classHistory withCriteria: changeCriteria numberOfVersions: numberOfVersions 
	| versionsChangesArray versionNum |
	versionsChangesArray := Array new: numberOfVersions.
	versionNum := 1.
	classHistory versions
		do: [:classVersion | 
			| previousClassVersion diffValue value1 value2 |
			versionNum := versionNum + 1.
			previousClassVersion := classVersion previousVersion.
			previousClassVersion isNil
				ifTrue: [diffValue := changeCriteria getNumber: classVersion]
				ifFalse: [value1 := changeCriteria getNumber: classVersion.
					value2 := changeCriteria getNumber: previousClassVersion.
					diffValue := (value1 - value2) abs].
			versionNum > 1
				ifTrue: [versionsChangesArray at: versionNum - 1 put: diffValue]].
	^ versionsChangesArray! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionChanges: hismoModel withCriteria: changeCriteria 
	| classesChangesDict classesHistories versionsNum |
	classesChangesDict := Dictionary new.
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not & each isRemoved not].
	versionsNum := hismoModel versions size.
	classesHistories
		do: [:classHistory | 
			| className changesArray |
			className := classHistory name.
			changesArray := self
						calculateClassVersionChanges: classHistory
						withCriteria: changeCriteria
						numberOfVersions: versionsNum.
			classesChangesDict at: className put: changesArray].
	^ classesChangesDict! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
createClassChangesLineChart: changesArray 
	| chart |
	chart := ESDiagramRenderer new.
	chart lineDiagram y: #changeNum;
		 yAxisLabel: 'Change Number';
		 identifier: #versionNum;
		 rotatedLabels: true;
		 regularAxis;
		 defaultColor: Color blue;
		
		preferredAxisMinY: (changesArray minValue: #changeNum);
		 models: changesArray.
	^ chart open! !

Object subclass: #VersionCriteriaCalculator
	instanceVariableNames: 'classesCriteriaDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
addClassesCriteria: newClassesCriteriaDict 
	newClassesCriteriaDict
		keysDo: [:className | | mergedVersionsInfo | (classesCriteriaDict includesKey: className)
				ifTrue: [mergedVersionsInfo := OrderedCollection new.
					mergedVersionsInfo
						addAll: (classesCriteriaDict at: className).
					mergedVersionsInfo
						addAll: (newClassesCriteriaDict at: className).
					classesCriteriaDict removeKey: className.
					classesCriteriaDict at: className put: mergedVersionsInfo sort]
				ifFalse: [classesCriteriaDict
						at: className
						put: (newClassesCriteriaDict at: className)]].
	^ classesCriteriaDict! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
addClassesCriteriaFromFile: fileName 
	| stream fileClassesCritDict lineAsArray versionsInfo className |
	fileClassesCritDict := Dictionary new.
	stream := FileStream readOnlyFileNamed: fileName.
	Transcript show: 'Adding classes criterias from file ' , fileName , ' ...';
		 cr;
		 cr.
	[stream atEnd]
		whileFalse: [lineAsArray := stream nextLine findBetweenSubStrs: '='.
			className := lineAsArray at: 1.
			versionsInfo := ((lineAsArray at: 2)
						findBetweenSubStrs: ',')
						collect: [:value | 
							| splitValue versInfo |
							splitValue := value findBetweenSubStrs: '-'.
							versInfo := VersionInfo new.
							versInfo
								versionNum: (splitValue at: 1).
							versInfo
								value: (splitValue at: 2)].
			fileClassesCritDict at: className put: versionsInfo].
	stream close.
	^ self addClassesCriteria: fileClassesCritDict! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionCriteria: classHistory withCriteria: changeCriteria 
	| versionsCriteriaList |
	versionsCriteriaList := LinkedList new.
	classHistory versions
		do: [:classVersion | 
			| criteriaValue versionInfo |
			criteriaValue := changeCriteria getNumber: classVersion.
			versionInfo := VersionInfo new.
			versionInfo
				versionNum: ((classVersion name findBetweenSubStrs: '.')
						at: 1).
			versionInfo value: criteriaValue.
			versionsCriteriaList add: versionInfo].
	^ versionsCriteriaList! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: changeCriteria fromHismoModel: hismoModel 
	| classesHistories newClassesCriteriaDict |
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not & each isRemoved not].
	classesHistories
		do: [:classHistory | 
			| className criteriaArray |
			className := classHistory name.
			criteriaArray := self calculateClassVersionCriteria: classHistory withCriteria: changeCriteria.
			newClassesCriteriaDict at: className put: criteriaArray].
	self addClassesCriteria: newClassesCriteriaDict.
	^ classesCriteriaDict! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: changeCriteria fromMSEFiles: mseFilesDirPath numOfFilesPerHismo: numFilesPerHismo 
	| mseFilesDir mseFiles mseFilesForCurrentHismo filesCount |
	Transcript show: 'Calculating classes criterias from the MSE files in ' , mseFilesDirPath;
		 cr.
	mseFilesDir := FileDirectory new setPathName: mseFilesDirPath.
	mseFiles := self orderMSEFiles: mseFilesDir.
	filesCount := 0.
	mseFilesForCurrentHismo := LinkedList new.
	mseFiles
		do: [:mseFile | 
			| currentHismo |
			filesCount := filesCount + 1.
			filesCount <= numFilesPerHismo
				ifTrue: [mseFilesForCurrentHismo add: mseFile]
				ifFalse: [Transcript show: 'Creating Hismo Model from the MSE files';
						 cr.
					currentHismo := self createHismoModelFromMSE: mseFilesForCurrentHismo.
					Transcript show: 'Calculating classes version criterias from the MSE files';
						 cr.
					self calculateClassesVersionCriteria: changeCriteria fromHismoModel: currentHismo.
					currentHismo remove.
					filesCount := 0]]! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
createHismoModelFromMSE: mseFiles 
	| hismoModel |
	hismoModel := HismoModelHistory new.
	mseFiles
		do: [:mseFile | 
			| stream mooseModel |
			stream := mseFile readStream.
			mooseModel := MooseModel new importFromMSEStream: stream.
			mooseModel
				name: ((mseFile name subStrings: '.')
						at: 1).
			hismoModel addLastVersionForMooseModel: mooseModel].
	^ hismoModel! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
initialize
	classesCriteriaDict := Dictionary new! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
orderMSEFiles: mseFilesDir 
	| mseFiles |
	mseFiles := SortedCollection
				sortBlock: [:c1 :c2 | ((c1 name subStrings: '.')
						at: 1) asInteger
						<= ((c2 name subStrings: '.')
								at: 1)].
	mseFiles addAll: mseFilesDir entries.
	^ mseFiles! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveClassesCriteriaToBinaryFile: fileName 
	| stream values |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Saving classes criterias to a binary file ' , fileName , ' ...';
		 cr;
		 cr.
	classesCriteriaDict
		keysDo: [:className | 
			stream binary; nextPutAll: className asByteArray.
			values := classesCriteriaDict at: className.
			stream binary; nextPutAll: values asByteArray].
	stream close! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveClassesCriteriaToFile: fileName 
	| stream versionsInfo |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Saving classes criterias to file ' , fileName , ' ...';
		 cr;
		 cr.
	classesCriteriaDict
		keysDo: [:className | 
			stream nextPutAll: className , '='.
			versionsInfo := classesCriteriaDict at: className.
			versionsInfo
				do: [:versInfo | 
					stream nextPutAll: versInfo versionNum asString , '-' , versInfo value asString.
					versionsInfo last = versInfo
						ifFalse: [stream nextPutAll: ',']].
			stream nextPutAll: '';
				 cr].
	stream close! !

Object subclass: #VersionInfo
	instanceVariableNames: 'versionNum value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
< vInfo 
	^ versionNum < vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
<= vInfo 
	^ versionNum <= vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
= vInfo 
	^ versionNum = vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
> vInfo 
	^ versionNum > vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
>= vInfo 
	^ versionNum >= vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
value
	^ value! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
value: val 
	value := val! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum
	^ versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum: num 
	versionNum := num! !

Object subclass: #VolatilityCalculator
	instanceVariableNames: 'sum sum2 num classesVolatilities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVolatilities: classesDict 
	| volatility |
	Transcript show: 'Calculating Volatility...';
		 cr;
		 cr.
	classesDict
		keysDo: [:className | 
			| classValues classVolatility |
			classValues := classesDict at: className.
			volatility := self calculateVolatility: classValues.
			classVolatility := ClassVolatility new.
			classVolatility className: className.
			classVolatility volatility: volatility.
			classesVolatilities add: classVolatility].
	classesVolatilities := classesVolatilities sort reverse.
	^ classesVolatilities! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVolatilities: classesDict saveToFile: fileName 
	| stream volatility |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Calculating Volatility...';
		 cr;
		 cr.
	classesDict
		keysDo: [:className | 
			| classValues versionNumsList valuesList |
			stream nextPutAll: 'Class Name: ' , className;
				 cr.
			classValues := classesDict at: className.
			versionNumsList := LinkedList new.
			valuesList := LinkedList new.
			classValues
				do: [:versionInfo | 
					versionNumsList add: versionInfo versionNum.
					valuesList add: versionInfo value].
			stream nextPutAll: 'Version Nums: ' , versionNumsList asCommaString;
				 cr.
			stream nextPutAll: 'Values: ' , valuesList asCommaString;
				 cr.
			volatility := self calculateVolatility: classValues.
			stream nextPutAll: 'Volatility: ' , volatility asString;
				 cr;
				 cr].
	stream close! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMean
	num > 0
		ifTrue: [^ sum / num].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateStdDev
	^ self calculateVariance sqrt! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVariance
	| mean |
	mean := self calculateMean.
	num > 0
		ifTrue: [^ sum2 / num - (mean * mean)].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVolatility: versionsInfo 
	| revisionsNum volatility stdDev |
	sum := 0.
	sum2 := 0.
	num := 0.
	versionsInfo
		do: [:versInfo | 
			| value |
			value := versInfo value.
			sum := sum + value.
			sum2 := sum2 + (value * value).
			num := num + 1.
			stdDev := self calculateStdDev].
	revisionsNum := versionsInfo size.
	^ stdDev * revisionsNum sqrt! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
initialize
	classesVolatilities := OrderedCollection new! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveClassesVolatilitiesToFile: fileName 
	| stream |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Saving Volatility to file ' , fileName , ' ...';
		 cr;
		 cr.
	classesVolatilities
		do: [:classVol | stream nextPutAll: classVol className , ' - ' , classVol volatility asString;
				 cr].
	stream close! !
